<?php

	/**
	 * @package content
	 */

	/**
	 * Developers can create new Frontend pages from this class. It provides
	 * an index view of all the pages in this Symphony install as well as the
	 * forms for the creation/editing of a Page
	 */
	require_once(TOOLKIT . '/class.administrationpage.php');
	require_once(TOOLKIT . '/class.resourcemanager.php');
	require_once(TOOLKIT . '/class.xsltprocess.php');

	class contentBlueprintsPages extends AdministrationPage {

		public $_errors = array();
		protected $_hilights = array();

		public function insertBreadcrumbsUsingPageIdentifier($page_id, $preserve_last = true) {
			if($page_id == 0) {
				return parent::insertBreadcrumbs(
					array(Widget::Anchor(__('Pages'), SYMPHONY_URL . '/blueprints/pages/'))
				);
			}

			$pages = PageManager::resolvePage($page_id, 'handle');

			foreach($pages as &$page){
				// If we are viewing the Page Editor, the Breadcrumbs should link
				// to the parent's Page Editor.
				if($this->_context[0] == 'edit') {
					$page = Widget::Anchor(
						PageManager::fetchTitleFromHandle($page),
						SYMPHONY_URL . '/blueprints/pages/edit/' . PageManager::fetchIDFromHandle($page) . '/'
					);
				}

				// If the pages index is nested, the Breadcrumb should link to the
				// Pages Index filtered by parent
				else if(Symphony::Configuration()->get('pages_table_nest_children', 'symphony') == 'yes') {
					$page = Widget::Anchor(
						PageManager::fetchTitleFromHandle($page),
						SYMPHONY_URL . '/blueprints/pages/?parent=' . PageManager::fetchIDFromHandle($page)
					);
				}

				// If there is no nesting on the Pages Index, the breadcrumb is
				// not a link, just plain text
				else {
					$page = new XMLElement('span', PageManager::fetchTitleFromHandle($page));
				}
			}

			if(!$preserve_last) array_pop($pages);

			parent::insertBreadcrumbs(array_merge(
				array(Widget::Anchor(__('Pages'), SYMPHONY_URL . '/blueprints/pages/')),
				$pages
			));
		}

		public function listAllPages($separator = '/') {
			$pages = PageManager::fetch(false, array('id', 'handle', 'title', 'path'));

			foreach($pages as &$page){
				$parents = explode('/', $page['path']);

				foreach($parents as &$parent){
					$parent = PageManager::fetchTitleFromHandle($parent);
				}

				$parents = implode($separator, $parents);
				$page['title'] = ($parents ? $parents . $separator . $page['title'] : $page['title']);
			}

			return $pages;
		}

		public function __viewIndex() {
			$this->setPageType('table');
			$this->setTitle(__('%1$s &ndash; %2$s', array(__('Pages'), __('Symphony'))));

			$nesting = (Symphony::Configuration()->get('pages_table_nest_children', 'symphony') == 'yes');

			if($nesting == true && isset($_GET['parent']) && is_numeric($_GET['parent'])) {
				$parent = PageManager::fetchPageByID((int)$_GET['parent'], array('title', 'id'));
			}

			$this->appendSubheading(isset($parent) ? $parent['title'] : __('Pages'), Widget::Anchor(
				__('Create New'), Administration::instance()->getCurrentPageURL() . 'new/' . ($nesting == true && isset($parent) ? "?parent={$parent['id']}" : NULL),
				__('Create a new page'), 'create button', NULL, array('accesskey' => 'c')
			));

			if(isset($parent)) {
				$this->insertBreadcrumbsUsingPageIdentifier($parent['id'], false);
			}

			$aTableHead = array(
				array(__('Title'), 'col'),
				array(__('Template'), 'col'),
				array('<abbr title="' . __('Universal Resource Locator') . '">' . __('URL') . '</abbr>', 'col'),
				array('<abbr title="' . __('Universal Resource Locator') . '">' . __('URL') . '</abbr> ' . __('Parameters'), 'col'),
				array(__('Type'), 'col')
			);
			$aTableBody = array();

			if($nesting == true){
				$aTableHead[] = array(__('Children'), 'col');
				$where = array(
					'parent ' . (isset($parent) ? " = {$parent['id']} " : ' IS NULL ')
				);
			}
			else {
				$where = array();
			}

			$pages = PageManager::fetch(true, array('*'), $where);

			if(!is_array($pages) or empty($pages)) {
				$aTableBody = array(Widget::TableRow(array(
					Widget::TableData(__('None found.'), 'inactive', null, count($aTableHead))
				), 'odd'));

			}
			else{
				foreach ($pages as $page) {
					$class = array();

					$page_title = ($nesting == true ? $page['title'] : PageManager::resolvePageTitle($page['id']));
					$page_url = URL . '/' . PageManager::resolvePagePath($page['id']) . '/';
					$page_edit_url = Administration::instance()->getCurrentPageURL() . 'edit/' . $page['id'] . '/';
					$page_template = PageManager::createFilePath($page['path'], $page['handle']);
					$page_template_url = Administration::instance()->getCurrentPageURL() . 'template/' . $page_template . '/';

					$col_title = Widget::TableData(Widget::Anchor(
						$page_title, $page_edit_url, $page['handle']
					));
					$col_title->appendChild(Widget::Input("items[{$page['id']}]", null, 'checkbox'));

					$col_template = Widget::TableData(Widget::Anchor(
						$page_template . '.xsl',
						$page_template_url
					));

					$col_url = Widget::TableData(Widget::Anchor($page_url, $page_url));

					if($page['params']) {
						$col_params = Widget::TableData(trim($page['params'], '/'));

					} else {
						$col_params = Widget::TableData(__('None'), 'inactive');
					}

					if(!empty($page['type'])) {
						$col_types = Widget::TableData(implode(', ', $page['type']));

					} else {
						$col_types = Widget::TableData(__('None'), 'inactive');
					}

					if(in_array($page['id'], $this->_hilights)) $class[] = 'failed';

					$columns = array($col_title, $col_template, $col_url, $col_params, $col_types);

					if($nesting == true){
						if(PageManager::hasChildPages($page['id'])){
							$col_children = Widget::TableData(
								Widget::Anchor(PageManager::getChildPagesCount($page['id']) . ' &rarr;',
								SYMPHONY_URL . '/blueprints/pages/?parent=' . $page['id'])
							);
						}
						else{
							$col_children = Widget::TableData(__('None'), 'inactive');
						}

						$columns[] = $col_children;
					}

					$aTableBody[] = Widget::TableRow(
						$columns,
						implode(' ', $class)
					);
				}
			}

			$table = Widget::Table(
				Widget::TableHead($aTableHead), null,
				Widget::TableBody($aTableBody), 'orderable selectable'
			);

			$this->Form->appendChild($table);

			$tableActions = new XMLElement('div');
			$tableActions->setAttribute('class', 'actions');

			$options = array(
				array(null, false, __('With Selected...')),
				array('delete', false, __('Delete'), 'confirm', null, array(
					'data-message' => __('Are you sure you want to delete the selected pages?')
				))
			);

			$tableActions->appendChild(Widget::Apply($options));
			$this->Form->appendChild($tableActions);
		}

		public function __viewTemplate() {
			$this->setPageType('form');
			$this->Form->setAttribute('action', SYMPHONY_URL . '/blueprints/pages/template/' . $this->_context[1] . '/');
			$this->Form->setAttribute('class', 'columns');

			$filename = $this->_context[1] . '.xsl';
			$file_abs = PAGES . '/' . $filename;

			$is_child = strrpos($this->_context[1],'_');
			$pagename = ($is_child != false ? substr($this->_context[1], $is_child + 1) : $this->_context[1]);
			$pagedata = PageManager::fetch(false, array('id'), array(
				"p.handle = '{$pagename}'"
			));
			$pagedata = array_pop($pagedata);

			if(!is_file($file_abs)) redirect(SYMPHONY_URL . '/blueprints/pages/');

			$fields['body'] = @file_get_contents($file_abs);

			$formHasErrors = (is_array($this->_errors) && !empty($this->_errors));
			if($formHasErrors) {
				$this->pageAlert(
					__('An error occurred while processing this form. See below for details.')
					, Alert::ERROR
				);
			}
			// These alerts are only valid if the form doesn't have errors
			else if(isset($this->_context[2])) {
				$this->pageAlert(
					__('Page updated at %s.', array(DateTimeObj::getTimeAgo()))
					. ' <a href="' . SYMPHONY_URL . '/blueprints/pages/new/" accesskey="c">'
					. __('Create another?')
					. '</a> <a href="' . SYMPHONY_URL . '/blueprints/pages/" accesskey="a">'
					. __('View all Pages')
					. '</a>'
					, Alert::SUCCESS);
			}

			$this->setTitle(__(
				($filename ? '%1$s &ndash; %2$s &ndash; %3$s' : '%2$s &ndash; %3$s'),
				array(
					$filename,
					__('Pages'),
					__('Symphony')
				)
			));

			$this->appendSubheading(__($filename ? $filename : __('Untitled')), Widget::Anchor(__('Edit Page'), SYMPHONY_URL . '/blueprints/pages/edit/' . $pagedata['id'] . '/', __('Edit Page Configuration'), 'button', NULL, array('accesskey' => 't')));
			$this->insertBreadcrumbsUsingPageIdentifier($pagedata['id']);

			if(!empty($_POST)) $fields = $_POST['fields'];

			$fields['body'] = htmlentities($fields['body'], ENT_COMPAT, 'UTF-8');

			$fieldset = new XMLElement('fieldset');
			$fieldset->setAttribute('class', 'primary column');

			$label = Widget::Label(__('Body'));
			$label->appendChild(Widget::Textarea(
				'fields[body]', 30, 80, $fields['body'],
				array(
					'class' => 'code'
				)
			));

			if(isset($this->_errors['body'])) {
				$label = Widget::Error($label, $this->_errors['body']);
			}

			$fieldset->appendChild($label);
			$this->Form->appendChild($fieldset);

			$utilities = General::listStructure(UTILITIES, array('xsl'), false, 'asc', UTILITIES);
			$utilities = $utilities['filelist'];

			if(is_array($utilities) && !empty($utilities)) {
				$this->Form->setAttribute('class', 'two columns');

				$div = new XMLElement('div');
				$div->setAttribute('class', 'secondary column');

				$p = new XMLElement('p', __('Utilities'));
				$p->setAttribute('class', 'label');
				$div->appendChild($p);

				$frame = new XMLElement('div', null, array('class' => 'frame'));

				$ul = new XMLElement('ul');
				$ul->setAttribute('id', 'utilities');

				foreach ($utilities as $util) {
					$li = new XMLElement('li');
					$li->appendChild(Widget::Anchor($util, SYMPHONY_URL . '/blueprints/utilities/edit/' . str_replace('.xsl', '', $util) . '/', NULL));
					$ul->appendChild($li);
				}

				$frame->appendChild($ul);
				$div->appendChild($frame);
				$this->Form->appendChild($div);
			}

			$div = new XMLElement('div');
			$div->setAttribute('class', 'actions');
			$div->appendChild(Widget::Input(
				'action[save]', __('Save Changes'),
				'submit', array('accesskey' => 's')
			));

			$this->Form->appendChild($div);
		}

		public function __viewNew() {
			$this->__viewEdit();
		}

		public function __viewEdit() {
			$this->setPageType('form');
			$fields = array();

			$nesting = (Symphony::Configuration()->get('pages_table_nest_children', 'symphony') == 'yes');

			// Verify page exists:
			if($this->_context[0] == 'edit') {
				if(!$page_id = $this->_context[1]) {
					redirect(SYMPHONY_URL . '/blueprints/pages/');
				}

				$existing = PageManager::fetchPageByID($page_id);

				if(!$existing) {
					Administration::instance()->errorPageNotFound();
				}
				else {
					$existing['type'] = PageManager::fetchPageTypes($page_id);
				}
			}

			// Status message:
			if(isset($this->_context[2])){
				$flag = $this->_context[2];
				$link_suffix = '';

				if(isset($_REQUEST['parent']) && is_numeric($_REQUEST['parent'])){
					$link_suffix = "?parent=" . $_REQUEST['parent'];
				}

				else if($nesting == true && isset($existing) && !is_null($existing['parent'])){
					$link_suffix = '?parent=' . $existing['parent'];
				}

				switch($flag){

					case 'saved':
						$this->pageAlert(
							__('Page updated at %s.', array(DateTimeObj::getTimeAgo()))
							. ' <a href="' . SYMPHONY_URL . '/blueprints/pages/new/" accesskey="c">'
							. __('Create another?')
							. '</a> <a href="' . SYMPHONY_URL . '/blueprints/pages/" accesskey="a">'
							. __('View all Pages')
							. '</a>'
							, Alert::SUCCESS);

						break;

					case 'created':
						$this->pageAlert(
							__('Page created at %s.', array(DateTimeObj::getTimeAgo()))
							. ' <a href="' . SYMPHONY_URL . '/blueprints/pages/new/" accesskey="c">'
							. __('Create another?')
							. '</a> <a href="' . SYMPHONY_URL . '/blueprints/pages/" accesskey="a">'
							. __('View all Pages')
							. '</a>'
							, Alert::SUCCESS);

				}
			}

			// Find values:
			if(isset($_POST['fields'])) {
				$fields = $_POST['fields'];
			}

			elseif($this->_context[0] == 'edit') {
				$fields = $existing;

				if(!is_null($fields['type'])) {
					$fields['type'] = implode(', ', $fields['type']);
				}

				$fields['data_sources'] = preg_split('/,/i', $fields['data_sources'], -1, PREG_SPLIT_NO_EMPTY);
				$fields['events'] = preg_split('/,/i', $fields['events'], -1, PREG_SPLIT_NO_EMPTY);
			}

			elseif(isset($_REQUEST['parent']) && is_numeric($_REQUEST['parent'])){
				$fields['parent'] = $_REQUEST['parent'];
			}

			$title = $fields['title'];
			if(trim($title) == '') $title = $existing['title'];

			$this->setTitle(__(
				($title ? '%1$s &ndash; %2$s &ndash; %3$s' : '%2$s &ndash; %3$s'),
				array(
					$title,
					__('Pages'),
					__('Symphony')
				)
			));

			if($existing) {
				$template_name = $fields['handle'];
				$page_url = URL . '/' . PageManager::resolvePagePath($page_id) . '/';
				if($existing['parent']){
					$parents = PageManager::resolvePagePath($existing['parent']);
					$template_name = PageManager::createFilePath($parents, $fields['handle']);
				}

				$this->appendSubheading($title, array(
					Widget::Anchor(__('View Page'), $page_url, __('View Page on Frontend'), 'button', NULL, array('target' => '_blank', 'accesskey' => 'v')),
					Widget::Anchor(__('Edit Page Template'), SYMPHONY_URL . '/blueprints/pages/template/' . $template_name, __('Edit Page Template'), 'button', NULL, array('accesskey' => 't'))
				));
			}
			else {
				$this->appendSubheading(($title ? $title : __('Untitled')));
			}

			if(isset($this->_context[1])) {
				$this->insertBreadcrumbsUsingPageIdentifier($this->_context[1], false);
			}
			else {
				$this->insertBreadcrumbsUsingPageIdentifier((int)$_GET['parent'], true);
			}

		// Title --------------------------------------------------------------

			$fieldset = new XMLElement('fieldset');
			$fieldset->setAttribute('class', 'settings');
			$fieldset->appendChild(new XMLElement('legend', __('Page Settings')));

			$label = Widget::Label(__('Title'));
			$label->appendChild(Widget::Input(
				'fields[title]', General::sanitize($fields['title'])
			));

			if(isset($this->_errors['title'])) {
				$label = Widget::Error($label, $this->_errors['title']);
			}

			$fieldset->appendChild($label);

		// Handle -------------------------------------------------------------

			$group = new XMLElement('div');
			$group->setAttribute('class', 'two columns');
			$column = new XMLElement('div');
			$column->setAttribute('class', 'column');

			$label = Widget::Label(__('URL Handle'));
			$label->appendChild(Widget::Input(
				'fields[handle]', $fields['handle']
			));

			if(isset($this->_errors['handle'])) {
				$label = Widget::Error($label, $this->_errors['handle']);
			}

			$column->appendChild($label);

		// Parent ---------------------------------------------------------

			$label = Widget::Label(__('Parent Page'));

			$where = array(
				sprintf('id != %d', $page_id)
			);
			$pages = PageManager::fetch(false, array('id'), $where, 'title ASC');

			$options = array(
				array('', false, '/')
			);

			if(!empty($pages)) {
				if(!function_exists('__compare_pages')) {
					function __compare_pages($a, $b) {
						return strnatcasecmp($a[2], $b[2]);
					}
				}

				foreach ($pages as $page) {
					$options[] = array(
						$page['id'], $fields['parent'] == $page['id'],
						'/' . PageManager::resolvePagePath($page['id'])
					);
				}

				usort($options, '__compare_pages');
			}

			$label->appendChild(Widget::Select(
				'fields[parent]', $options
			));
			$column->appendChild($label);
			$group->appendChild($column);

		// Parameters ---------------------------------------------------------

			$column = new XMLElement('div');
			$column->setAttribute('class', 'column');

			$label = Widget::Label(__('URL Parameters'));
			$label->appendChild(Widget::Input(
				'fields[params]', $fields['params'], 'text', array('placeholder' => 'param1/param2')
			));
			$column->appendChild($label);

		// Type -----------------------------------------------------------

			$label = Widget::Label(__('Page Type'));
			$label->appendChild(Widget::Input('fields[type]', $fields['type']));

			if(isset($this->_errors['type'])) {
				$label = Widget::Error($label, $this->_errors['type']);
			}

			$column->appendChild($label);

			$tags = new XMLElement('ul');
			$tags->setAttribute('class', 'tags');

			$types = PageManager::fetchAvailablePageTypes();
			foreach($types as $type) {
				$tags->appendChild(new XMLElement('li', $type));
			}

			$column->appendChild($tags);
			$group->appendChild($column);
			$fieldset->appendChild($group);
			$this->Form->appendChild($fieldset);

		// Events -------------------------------------------------------------

			$fieldset = new XMLElement('fieldset');
			$fieldset->setAttribute('class', 'settings');
			$fieldset->appendChild(new XMLElement('legend', __('Page Resources')));

			$group = new XMLElement('div');
			$group->setAttribute('class', 'two columns');

			$label = Widget::Label(__('Events'));
			$label->setAttribute('class', 'column');

			$events = ResourceManager::fetch(RESOURCE_TYPE_EVENT, array(), array(), 'name ASC');
			$options = array();

			if(is_array($events) && !empty($events)) {
				if(!is_array($fields['events'])) $fields['events'] = array();
				foreach ($events as $name => $about) $options[] = array(
					$name, in_array($name, $fields['events']), $about['name']
				);
			}

			$label->appendChild(Widget::Select('fields[events][]', $options, array('multiple' => 'multiple')));
			$group->appendChild($label);

		// Data Sources -------------------------------------------------------

			$label = Widget::Label(__('Data Sources'));
			$label->setAttribute('class', 'column');

			$datasources = ResourceManager::fetch(RESOURCE_TYPE_DS, array(), array(), 'name ASC');
			$options = array();

			if(is_array($datasources) && !empty($datasources)) {
				if(!is_array($fields['data_sources'])) $fields['data_sources'] = array();
				foreach ($datasources as $name => $about) $options[] = array(
					$name, in_array($name, $fields['data_sources']), $about['name']
				);
			}

			$label->appendChild(Widget::Select('fields[data_sources][]', $options, array('multiple' => 'multiple')));
			$group->appendChild($label);
			$fieldset->appendChild($group);
			$this->Form->appendChild($fieldset);

		// Controls -----------------------------------------------------------

			/**
			 * After all Page related Fields have been added to the DOM, just before the
			 * actions.
			 *
			 * @delegate AppendPageContent
			 * @param string $context
			 *  '/blueprints/pages/'
			 * @param XMLElement $form
			 * @param array $fields
			 * @param array $errors
			 */
			Symphony::ExtensionManager()->notifyMembers(
				'AppendPageContent',
				'/blueprints/pages/',
				array(
					'form'		=> &$this->Form,
					'fields'	=> &$fields,
					'errors'	=> $this->_errors
				)
			);

			$div = new XMLElement('div');
			$div->setAttribute('class', 'actions');
			$div->appendChild(Widget::Input(
				'action[save]', ($this->_context[0] == 'edit' ? __('Save Changes') : __('Create Page')),
				'submit', array('accesskey' => 's')
			));

			if($this->_context[0] == 'edit'){
				$button = new XMLElement('button', __('Delete'));
				$button->setAttributeArray(array('name' => 'action[delete]', 'class' => 'button confirm delete', 'title' => __('Delete this page'), 'accesskey' => 'd', 'data-message' => __('Are you sure you want to delete this page?')));
				$div->appendChild($button);
			}

			$this->Form->appendChild($div);

			if(isset($_REQUEST['parent']) && is_numeric($_REQUEST['parent'])){
				$this->Form->appendChild(new XMLElement('input', NULL, array('type' => 'hidden', 'name' => 'parent', 'value' => $_REQUEST['parent'])));
			}
		}

		public function __actionIndex() {
			$checked = (is_array($_POST['items'])) ? array_keys($_POST['items']) : null;

			if(is_array($checked) && !empty($checked)) {
				switch ($_POST['with-selected']) {
					case 'delete':
						$this->__actionDelete($checked, SYMPHONY_URL . '/blueprints/pages/');
						break;
				}
			}
		}

		public function __actionTemplate() {
			$filename = $this->_context[1] . '.xsl';
			$file_abs = PAGES . '/' . $filename;
			$fields = $_POST['fields'];
			$this->_errors = array();

			if(!isset($fields['body']) || trim($fields['body']) == '') {
				$this->_errors['body'] = __('This is a required field.');
			}
			else if(!General::validateXML($fields['body'], $errors, false, new XSLTProcess())) {
				$this->_errors['body'] = __('This document is not well formed.') . ' ' . __('The following error was returned:') . ' <code>' . $errors[0]['message'] . '</code>';
			}

			if(empty($this->_errors)) {
				/**
				 * Just before a Page Template is about to written to disk
				 *
				 * @delegate PageTemplatePreEdit
				 * @since Symphony 2.2.2
				 * @param string $context
				 * '/blueprints/pages/template/'
				 * @param string $file
				 *  The path to the Page Template file
				 * @param string $contents
				 *  The contents of the `$fields['body']`, passed by reference
				 */
				Symphony::ExtensionManager()->notifyMembers('PageTemplatePreEdit', '/blueprints/pages/template/', array('file' => $file_abs, 'contents' => &$fields['body']));

				if(!PageManager::writePageFiles($file_abs, $fields['body'])) {
					$this->pageAlert(
						__('Page Template could not be written to disk.')
						. ' ' . __('Please check permissions on %s.', array('<code>/workspace/pages</code>'))
						, Alert::ERROR
					);

				}
				else {
					/**
					 * Just after a Page Template has been edited and written to disk
					 *
					 * @delegate PageTemplatePostEdit
					 * @since Symphony 2.2.2
					 * @param string $context
					 * '/blueprints/pages/template/'
					 * @param string $file
					 *  The path to the Page Template file
					 */
					Symphony::ExtensionManager()->notifyMembers('PageTemplatePostEdit', '/blueprints/pages/template/', array('file' => $file_abs));

					redirect(SYMPHONY_URL . '/blueprints/pages/template/' . $this->_context[1] . '/saved/');
				}
			}
		}

		public function __actionNew() {
			$this->__actionEdit();
		}

		public function __actionEdit() {
			if($this->_context[0] != 'new' && !$page_id = (integer)$this->_context[1]) {
				redirect(SYMPHONY_URL . '/blueprints/pages/');
			}

			$parent_link_suffix = NULL;
			if(isset($_REQUEST['parent']) && is_numeric($_REQUEST['parent'])){
				$parent_link_suffix = '?parent=' . $_REQUEST['parent'];
			}

			if(@array_key_exists('delete', $_POST['action'])) {
				$this->__actionDelete($page_id, SYMPHONY_URL  . '/blueprints/pages/' . $parent_link_suffix);
			}

			if(@array_key_exists('save', $_POST['action'])) {

				$fields = $_POST['fields'];
				$this->_errors = array();
				$autogenerated_handle = false;

				if(!isset($fields['title']) || trim($fields['title']) == '') {
					$this->_errors['title'] = __('This is a required field');
				}

				if(trim($fields['type']) != '' && preg_match('/(index|404|403)/i', $fields['type'])) {
					$types = preg_split('/\s*,\s*/', strtolower($fields['type']), -1, PREG_SPLIT_NO_EMPTY);

					if(in_array('index', $types) && PageManager::hasPageTypeBeenUsed($page_id, 'index')) {
						$this->_errors['type'] = __('An index type page already exists.');
					}

					elseif(in_array('404', $types) && PageManager::hasPageTypeBeenUsed($page_id, '404')) {
						$this->_errors['type'] = __('A 404 type page already exists.');
					}

					elseif(in_array('403', $types) && PageManager::hasPageTypeBeenUsed($page_id, '403')) {
						$this->_errors['type'] = __('A 403 type page already exists.');
					}
				}

				if(trim($fields['handle'] ) == '') {
					$fields['handle'] = $fields['title'];
					$autogenerated_handle = true;
				}

				$fields['handle'] = PageManager::createHandle($fields['handle']);
				if(empty($fields['handle']) && !isset($this->_errors['title'])) {
					$this->_errors['handle'] = __('Please ensure handle contains at least one Latin-based character.');
				}

				/**
				 * Just after the Symphony validation has run, allows Developers
				 * to run custom validation logic on a Page
				 *
				 * @delegate PagePostValidate
				 * @since Symphony 2.2
				 * @param string $context
				 * '/blueprints/pages/'
				 * @param array $fields
				 *  The `$_POST['fields']` array. This should be read-only and not changed
				 *  through this delegate.
				 * @param array $errors
				 *  An associative array of errors, with the key matching a key in the
				 *  `$fields` array, and the value being the string of the error. `$errors`
				 *  is passed by reference.
				 */
				Symphony::ExtensionManager()->notifyMembers('PagePostValidate', '/blueprints/pages/', array('fields' => $fields, 'errors' => &$errors));

				if(empty($this->_errors)) {

					$autogenerated_handle = false;

					if($fields['params']) {
						$fields['params'] = trim(preg_replace('@\/{2,}@', '/', $fields['params']), '/');
					}

					// Clean up type list
					$types = preg_split('/\s*,\s*/', $fields['type'], -1, PREG_SPLIT_NO_EMPTY);
					$types = @array_map('trim', $types);
					unset($fields['type']);

					$fields['parent'] = ($fields['parent'] != __('None') ? $fields['parent'] : null);
					$fields['data_sources'] = is_array($fields['data_sources']) ? implode(',', $fields['data_sources']) : NULL;
					$fields['events'] = is_array($fields['events']) ? implode(',', $fields['events']) : NULL;
					$fields['path'] = null;

					if($fields['parent']) {
						$fields['path'] = PageManager::resolvePagePath((integer)$fields['parent']);
					}

					// Check for duplicates:
					$current = PageManager::fetchPageByID($page_id);

					if(empty($current)) {
						$fields['sortorder'] = PageManager::fetchNextSortOrder();
					}

					$where = array();

					if(!empty($current)) {
						$where[] = "p.id != {$page_id}";
					}
					$where[] = "p.handle = '" . $fields['handle'] . "'";
					$where[] = (is_null($fields['path']))
						? "p.path IS NULL"
						: "p.path = '" . $fields['path'] . "'";
					$duplicate = PageManager::fetch(false, array('*'), $where);

					// If duplicate
					if(!empty($duplicate)) {
						if($autogenerated_handle) {
							$this->_errors['title'] = __('A page with that title already exists');
						}
						else {
							$this->_errors['handle'] = __('A page with that handle already exists');
						}
					}
					// Create or move files:
					else {
						// New page?
						if(empty($current)) {
							$file_created = PageManager::createPageFiles(
								$fields['path'], $fields['handle']
							);
						}
						// Existing page, potentially rename files
						else {
							$file_created = PageManager::createPageFiles(
								$fields['path'], $fields['handle'],
								$current['path'], $current['handle']
							);
						}

						// If the file wasn't created, it's usually permissions related
						if(!$file_created) {
							$redirect = null;
							return $this->pageAlert(
								__('Page Template could not be written to disk.')
								. ' ' . __('Please check permissions on %s.', array('<code>/workspace/pages</code>'))
								, Alert::ERROR
							);
						}

						// Insert the new data:
						if(empty($current)) {

							/**
							 * Just prior to creating a new Page record in `tbl_pages`, provided
							 * with the `$fields` associative array. Use with caution, as no
							 * duplicate page checks are run after this delegate has fired
							 *
							 * @delegate PagePreCreate
							 * @since Symphony 2.2
							 * @param string $context
							 * '/blueprints/pages/'
							 * @param array $fields
							 *  The `$_POST['fields']` array passed by reference
							 */
							Symphony::ExtensionManager()->notifyMembers('PagePreCreate', '/blueprints/pages/', array('fields' => &$fields));

							if(!$page_id = PageManager::add($fields)) {
								$this->pageAlert(
									__('Unknown errors occurred while attempting to save.')
									. '<a href="' . SYMPHONY_URL . '/system/log/">'
									. __('Check your activity log')
									. '</a>.'
									, Alert::ERROR);

							}
							else {
								/**
								 * Just after the creation of a new page in `tbl_pages`
								 *
								 * @delegate PagePostCreate
								 * @since Symphony 2.2
								 * @param string $context
								 * '/blueprints/pages/'
								 * @param integer $page_id
								 *  The ID of the newly created Page
								 * @param array $fields
								 *  An associative array of data that was just saved for this page
								 */
								Symphony::ExtensionManager()->notifyMembers('PagePostCreate', '/blueprints/pages/', array('page_id' => $page_id, 'fields' => &$fields));

								$redirect = "/blueprints/pages/edit/{$page_id}/created/{$parent_link_suffix}";
							}

						}
						// Update existing:
						else {

							/**
							 * Just prior to updating a Page record in `tbl_pages`, provided
							 * with the `$fields` associative array. Use with caution, as no
							 * duplicate page checks are run after this delegate has fired
							 *
							 * @delegate PagePreEdit
							 * @since Symphony 2.2
							 * @param string $context
							 * '/blueprints/pages/'
							 * @param integer $page_id
							 *  The ID of the Page that is about to be updated
							 * @param array $fields
							 *  The `$_POST['fields']` array passed by reference
							 */
							Symphony::ExtensionManager()->notifyMembers('PagePreEdit', '/blueprints/pages/', array('page_id' => $page_id, 'fields' => &$fields));

							if(!PageManager::edit($page_id, $fields, true)) {
								return $this->pageAlert(
										__('Unknown errors occurred while attempting to save.')
										. '<a href="' . SYMPHONY_URL . '/system/log/">'
										. __('Check your activity log')
										. '</a>.'
										, Alert::ERROR);

							}
							else {
								/**
								 * Just after updating a page in `tbl_pages`
								 *
								 * @delegate PagePostEdit
								 * @since Symphony 2.2
								 * @param string $context
								 * '/blueprints/pages/'
								 * @param integer $page_id
								 *  The ID of the Page that was just updated
								 * @param array $fields
								 *  An associative array of data that was just saved for this page
								 */
								Symphony::ExtensionManager()->notifyMembers('PagePostEdit', '/blueprints/pages/', array('page_id' => $page_id, 'fields' => $fields));

								$redirect = "/blueprints/pages/edit/{$page_id}/saved/{$parent_link_suffix}";
							}
						}
					}

					// Only proceed if there was no errors saving/creating the page
					if(empty($this->_errors)) {
						/**
						 * Just before the page's types are saved into `tbl_pages_types`.
						 * Use with caution as no further processing is done on the `$types`
						 * array to prevent duplicate `$types` from occurring (ie. two index
						 * page types). Your logic can use the PageManger::hasPageTypeBeenUsed
						 * function to perform this logic.
						 *
						 * @delegate PageTypePreCreate
						 * @since Symphony 2.2
						 * @see toolkit.PageManager#hasPageTypeBeenUsed
						 * @param string $context
						 * '/blueprints/pages/'
						 * @param integer $page_id
						 *  The ID of the Page that was just created or updated
						 * @param array $types
						 *  An associative array of the types for this page passed by reference.
						 */
						Symphony::ExtensionManager()->notifyMembers('PageTypePreCreate', '/blueprints/pages/', array('page_id' => $page_id, 'types' => &$types));

						// Assign page types:
						PageManager::addPageTypesToPage($page_id, $types);

						// Find and update children:
						if($this->_context[0] == 'edit') {
							PageManager::editPageChildren($page_id, $fields['path'] . '/' . $fields['handle']);
						}

						if($redirect) redirect(SYMPHONY_URL . $redirect);
					}
				}

				// If there was any errors, either with pre processing or because of a
				// duplicate page, return.
				if(is_array($this->_errors) && !empty($this->_errors)) {
					return $this->pageAlert(
						__('An error occurred while processing this form. See below for details.')
						, Alert::ERROR
					);
				}
			}
		}

		public function __actionDelete($pages, $redirect) {
			$success = true;
			$deleted_page_ids = array();

			if(!is_array($pages)) $pages = array($pages);

			/**
			 * Prior to deleting Pages
			 *
			 * @delegate PagePreDelete
			 * @since Symphony 2.2
			 * @param string $context
			 * '/blueprints/pages/'
			 * @param array $page_ids
			 *  An array of Page ID's that are about to be deleted, passed
			 *  by reference
			 * @param string $redirect
			 *  The absolute path that the Developer will be redirected to
			 *  after the Pages are deleted
			 */
			Symphony::ExtensionManager()->notifyMembers('PagePreDelete', '/blueprints/pages/', array('page_ids' => &$pages, 'redirect' => &$redirect));

			foreach($pages as $page_id) {
				$page = PageManager::fetchPageByID($page_id);

				if(empty($page)) {
					$success = false;
					$this->pageAlert(
						__('Page could not be deleted because it does not exist.'),
						Alert::ERROR
					);

					break;
				}

				if(PageManager::hasChildPages($page_id)) {
					$this->_hilights[] = $page['id'];
					$success = false;
					$this->pageAlert(
						__('Page could not be deleted because it has children.'),
						Alert::ERROR
					);

					continue;
				}

				if(!PageManager::deletePageFiles($page['path'], $page['handle'])) {
					$this->_hilights[] = $page['id'];
					$success = false;
					$this->pageAlert(
						__('One or more pages could not be deleted.')
						. ' ' . __('Please check permissions on %s.', array('<code>/workspace/pages</code>'))
						, Alert::ERROR
					);

					continue;
				}

				if(PageManager::delete($page_id, false)) {
					$deleted_page_ids[] = $page_id;
				}
			}

			if($success) {
				/**
				 * Fires after all Pages have been deleted
				 *
				 * @delegate PagePostDelete
				 * @since Symphony 2.3
				 * @param string $context
				 * '/blueprints/pages/'
				 * @param array $page_ids
				 *  The page ID's that were just deleted
				 */
				Symphony::ExtensionManager()->notifyMembers('PagePostDelete', '/blueprints/pages/', array('page_ids' => $deleted_page_ids));
				redirect($redirect);
			}
		}

		/**
		 * Returns boolean if a the given `$type` is set for
		 * the given `$page_id`.
		 *
		 * @deprecated This will be removed in Symphony 2.4.
		 *  The preferred function is `PageManger::hasPageTypeBeenUsed`
		 * @see toolkit.PageManager#hasPageTypeBeenUsed
		 * @param integer $page_id
		 *  The ID of the Page to check
		 * @param string $type
		 * @return boolean
		 *  True if the type is used, false otherwise
		 */
		public static function typeUsed($page_id, $type) {
			return PageManager::hasPageTypeBeenUsed($page_id, $type);
		}
	}
